@@ -1036,6 +1036,49 @@ def str_equals(x, y):
     return equals_mask
 
 
+@register_function(scope='str')
+@auto_str_unwrap
+def str_notequals(x, y):
+    """Tests if strings x and y are the not same
+
+    :returns: a boolean expression
+
+    Example:
+
+    >>> import vaex
+    >>> text = ['Something', 'very pretty', 'is coming', 'our', 'way.']
+    >>> df = vaex.from_arrays(text=text)
+    >>> df
+      #  text
+      0  Something
+      1  very pretty
+      2  is coming
+      3  our
+      4  way.
+
+    >>> df.text.str.notequals(df.text)
+    Expression = str_notequals(text, text)
+    Length: 5 dtype: bool (expression)
+    ----------------------------------
+    0  False
+    1  False
+    2  False
+    3  False
+    4  False
+
+    >>> df.text.str.notequals('our')
+    Expression = str_notequals(text, 'our')
+    Length: 5 dtype: bool (expression)
+    ----------------------------------
+    0   True
+    1   True
+    2   True
+    3  False
+    4   True
+    """
+    return ~str_equals(x, y)
+
+
 @register_function(scope='str')
 @auto_str_unwrap
 def str_capitalize(x):
@@ -2485,10 +2528,17 @@ def _isin(x, values):
         # but numpy doesn't know what to do with that
         if hasattr(values, 'to_numpy'):
             values = values.to_numpy()
-        if np.ma.isMaskedArray(x):
-            return np.ma.isin(x, values)
+        mask = isnan(values)
+        if np.any(mask):
+            if np.ma.isMaskedArray(x):
+                return np.ma.isin(x, values) | isnan(x)
+            else:
+                return np.isin(x, values) | isnan(x)
         else:
-            return np.isin(x, values)
+            if np.ma.isMaskedArray(x):
+                return np.ma.isin(x, values)
+            else:
+                return np.isin(x, values)
 
 
 @register_function(name='isin_set', on_expression=False)
@@ -2561,7 +2611,7 @@ def add_geo_json(ds, json_or_file, column_name, longitude_expression, latitude_e
 import vaex.arrow.numpy_dispatch
 
 @register_function()
-def where(condition, x, y):
+def where(condition, x, y, dtype=None):
     # special where support for strings
     # TODO: this should be replaced by an arrow compute function in the future
     if type(x) == str:
@@ -2611,6 +2661,29 @@ def where(condition, x, y):
         choices = vaex.array_types.concat([x, y])
         values = choices.take(indices)
         return values
+
+    # cast x and y
+    if dtype is not None:
+        if np.can_cast(x, dtype):
+            dtype_scalar = np.dtype(dtype)
+            x = dtype_scalar.type(x)
+        if np.can_cast(y, dtype):
+            dtype_scalar = np.dtype(dtype)
+            y = dtype_scalar.type(y)
+
     # default callback is on numpy
+    # where() respects the dtypes of x and y; ex: if x and y are 'uint8', the resulting array will also be 'uint8'
     ar = np.where(condition, x, y)
+
     return ar
+
+
+@register_function()
+def index_values(ar):
+    dtype = vaex.dtype_of(ar)
+    if not dtype.is_encoded:
+        raise TypeError(f'Can only get index values from a (dictionary) encoded array, not for {ar}')
+    if isinstance(ar, pa.ChunkedArray):
+        return pa.chunked_array([k.indices for k in ar.chunks], type=ar.type.index_type)
+    else:
+        return ar.indices
